import unittest

from kanjize import kanji2int, int2kanji, Number


class TestKanjize(unittest.TestCase):
    """
    test class of kanjize
    """

    def test_kanji2int(self):
        self.assertEqual(1, kanji2int("一"))
        self.assertEqual(10, kanji2int("十"))
        self.assertEqual(11, kanji2int("十一"))
        self.assertEqual(111, kanji2int("百十一"))
        self.assertEqual(211, kanji2int("二百十一"))
        self.assertEqual(121, kanji2int("百二十一"))
        self.assertEqual(11, kanji2int("十一"))
        self.assertEqual(111, kanji2int("百十一"))
        self.assertEqual(211, kanji2int("二百十一"))
        self.assertEqual(121, kanji2int("百二十一"))
        self.assertEqual(1_000, kanji2int("千"))
        self.assertEqual(1_001, kanji2int("千一"))
        self.assertEqual(2_025, kanji2int("二千二十五"))
        self.assertEqual(58_076_099, kanji2int("五千八百七万六千九十九"))

        # added 1.0.0
        self.assertEqual(10_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000, kanji2int("一恒河沙"))
        self.assertEqual(
            999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999,
            kanji2int("九千九百九十九無量大数九千九百九十九不可思議九千九百九十九那由多九千九百九十九阿僧祇九千九百九十九恒河沙九千九百九十九極"
                      "九千九百九十九載九千九百九十九正九千九百九十九澗九千九百九十九溝九千九百九十九穣九千九百九十九𥝱九千九百九十九垓"
                      "九千九百九十九京九千九百九十九兆九千九百九十九億九千九百九十九万九千九百九十九")
        )

        self.assertEqual(1, kanji2int("1", style="mixed"))
        self.assertEqual(10, kanji2int("10", style="mixed"))
        self.assertEqual(11, kanji2int("11", style="mixed"))
        self.assertEqual(121, kanji2int("121", style="mixed"))
        self.assertEqual(1_000, kanji2int("1千", style="mixed"))
        self.assertEqual(1_001, kanji2int("1001", style="mixed"))
        self.assertEqual(2_025, kanji2int("2025", style="mixed"))
        self.assertEqual(58_076_099, kanji2int("5807万6099", style="mixed"))
        self.assertEqual(223_423_542_566_000, kanji2int("223兆4235億4256万6千", style="mixed"))
        self.assertEqual(223_400_042_566_000, kanji2int("223兆4千億4256万6千", style="mixed"))
        self.assertEqual(50_000_000_000_000_000_000, kanji2int("5千京", style="mixed"))
        self.assertEqual(394_385_000_048_950_000, kanji2int("39京4385兆4895万", style="mixed"))

        # added 0.2.1
        self.assertEqual(1_001, kanji2int("千1", style="mixed"))
        self.assertEqual(2_025, kanji2int("2千2十5", style="mixed"))
        self.assertEqual(4_000_000, kanji2int("4百万", style="mixed"))
        self.assertEqual(58_076_099, kanji2int("58百7万6千99", style="mixed"))
        self.assertEqual(22_342_354_2566_000, kanji2int("2百23兆4千2百3十5億4256万6千", style="mixed"))
        self.assertEqual(22_340_004_2566_000, kanji2int("223兆4千億4256万6千", style="mixed"))
        self.assertEqual(22_340_004_2566_000, kanji2int("2234千億4256万6千", style="mixed"))
        self.assertEqual(50_500_000_000_000_000_000, kanji2int("5千5十京", style="mixed"))
        self.assertEqual(394_385_000_048_950_000, kanji2int("39京4385兆4895万", style="mixed"))

        # added 1.0.0
        self.assertEqual(10_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000,
                         kanji2int("1恒河沙", style="mixed"))
        self.assertEqual(
            999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999,
            kanji2int("9999無量大数9999不可思議9999那由多9999阿僧祇"
                      "9999恒河沙9999極9999載9999正9999澗9999溝9999穣"
                      "9999𥝱9999垓9999京9999兆9999億9999万9999")
        )

        # added 1.1.0
        self.assertEqual(150_000_000, kanji2int("1.5億", style="mixed"))
        self.assertEqual(250_320, kanji2int("250.32千", style="mixed"))
        self.assertEqual(150_250_320, kanji2int("1.5億250.32千", style="mixed"))

    def test_int2kanji(self):
        self.assertEqual(int2kanji(1), "一", "all")
        self.assertEqual(int2kanji(10), "十", "all")
        self.assertEqual(int2kanji(11), "十一", "all")
        self.assertEqual(int2kanji(111), "百十一", "all")
        self.assertEqual(int2kanji(211), "二百十一", "all")
        self.assertEqual(int2kanji(121), "百二十一", "all")
        self.assertEqual(int2kanji(11), "十一", "all")
        self.assertEqual(int2kanji(111), "百十一", "all")
        self.assertEqual(int2kanji(211), "二百十一")
        self.assertEqual(int2kanji(121), "百二十一")
        self.assertEqual(int2kanji(1_000), "千")
        self.assertEqual(int2kanji(1_001), "千一")
        self.assertEqual(int2kanji(2_025), "二千二十五")
        self.assertEqual(int2kanji(58_076_099), "五千八百七万六千九十九")

        # added 1.0.0
        self.assertEqual(int2kanji(10_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000), "一恒河沙")
        self.assertEqual(
            int2kanji(999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999),
            ("九千九百九十九無量大数九千九百九十九不可思議九千九百九十九那由多九千九百九十九阿僧祇九千九百九十九恒河沙九千九百九十九極"
             "九千九百九十九載九千九百九十九正九千九百九十九澗九千九百九十九溝九千九百九十九穣九千九百九十九𥝱九千九百九十九垓"
             "九千九百九十九京九千九百九十九兆九千九百九十九億九千九百九十九万九千九百九十九")
        )

        self.assertEqual("1", int2kanji(1, style="mixed"))
        self.assertEqual("10", int2kanji(10, style="mixed"))
        self.assertEqual("11", int2kanji(11, style="mixed"))
        self.assertEqual("121", int2kanji(121, style="mixed"))
        self.assertEqual("1千", int2kanji(1_000, style="mixed"))
        self.assertEqual("1001", int2kanji(1_001, style="mixed"))
        self.assertEqual("2025", int2kanji(2_025, style="mixed"))
        self.assertEqual("5807万6099", int2kanji(58_076_099, style="mixed"))
        self.assertEqual("223兆4235億4256万6千", int2kanji(223_423_542_566_000, style="mixed"))
        self.assertEqual("5千京", int2kanji(50_000_000_000_000_000_000, style="mixed"))
        self.assertEqual("5000京", int2kanji(50_000_000_000_000_000_000, style="mixed", kanji_thousand=False))
        self.assertEqual("39京4385兆4895万", int2kanji(394_385_000_048_950_000, style="mixed"))
        self.assertEqual("223兆4千億4256万6千", int2kanji(223_400_042_566_000, style="mixed"))
        self.assertEqual("223兆4000億4256万6000", int2kanji(223_400_042_566_000, style="mixed", kanji_thousand=False))

        # added 1.0.0
        self.assertEqual("1恒河沙", int2kanji(10_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000_000,
                                           style="mixed",
                                           kanji_thousand=False))
        self.assertEqual(
            "9999無量大数9999不可思議9999那由多9999阿僧祇9999恒河沙9999極9999載9999正9999澗9999溝9999穣9999𥝱9999垓9999京9999兆9999億9999万9999",
            int2kanji(999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999_999,
                      style='mixed')
        )

    def test_number(self):
        self.assertEqual(12000, Number(276_493_734) - Number.from_kanji("2億7648万1734"))
        self.assertEqual(12734, Number(276_493_734) - Number.from_kanji("2億7648万1千"))
        self.assertEqual("2億7648万1734", (Number(276_493_734) - Number(12_000)).to_kanji(style="mixed"))


if __name__ == "__main__":
    unittest.main()



import math
import re


def int2kanji(number: int, error="raise", style="all", kanji_thousand=True) -> str:
    """
    :param number: Integer to convert into Kanji
    :param error: How to handle Error. "raise": raise error. "ignore": ignore error , "warn": warn but don't raise
    :param style: Which style of format will be used. "mixed": Arabic and Kanji Mixed like "4億5230万3千", "all": All letter will be Kanji.
    :param kanji_thousand: Whether you make a thousand to kanji. this will be used if style="mixed"
    :return: str
    """
    if error not in ("raise", "warn", "ignore"):
        raise ValueError("unexpected value {} for argument error".format(error))
    if style not in ("all", "mixed"):
        raise ValueError("unexpected value {} for argument style".format(style))  # check arguments

    kanji = {1: '一', 2: '二', 3: '三', 4: '四', 5: '五', 6: '六', 7: '七', 8: '八', 9: '九'}
    digits = ('', '万', '億', '兆', '京', '垓', '𥝱', '穣', '溝', '澗', '正', '載', '極', '恒河沙', '阿僧祇', '那由多', '不可思議', '無量大数')

    if style == "all":
        res = ""  # all letters will be added to this

        for i in range(math.ceil(math.log(number, 1000)), -1, -1):
            c_num = str((number % (10 ** ((i + 1) * 4))) // (10 ** (i * 4))).zfill(4)  # remainder
            c_str = ""
            if c_num == "0000":
                continue
            if c_num[0] > "0":  # 1st digit
                if c_num[0] != "1":
                    c_str += kanji[int(c_num[0])]
                c_str += "千"
            if c_num[1] > "0":  # 2nd digit
                if c_num[1] != "1":
                    c_str += kanji[int(c_num[1])]
                c_str += "百"
            if c_num[2] > "0":  # 3rd digit
                if c_num[2] != "1":
                    c_str += kanji[int(c_num[2])]
                c_str += "十"
            if c_num[3] > "0":  # 4th digit
                c_str += kanji[int(c_num[3])]
            if c_str:
                res += c_str + digits[i]
        return res

    elif style == "mixed":
        res = ""  # all letters will be added to this

        for i in range(math.ceil(math.log(number, 1000)), -1, -1):
            c_num = (number % (10 ** ((i + 1) * 4))) // (10 ** (i * 4))  # reminder
            c_str = ""
            if kanji_thousand and c_num // 1000 == c_num / 1000 and c_num // 1000:  # If number is n * thousand
                c_str += str(c_num).zfill(4)[-4] + "千"
            elif c_num:
                c_str = str(c_num)
            if c_str:
                res += c_str + digits[i]
        return res


def kanji2int(kanjis: str, error="raise", style="auto") -> int:
    """
    :param kanjis: Kanji str to convert into Integer
    :param error: How to handle Error. "raise": raise error. "ignore": ignore error , "warn": warn but don't raise
    :param style: Which style of format will be used. "mixed": Arabic and Kanji Mixed like "4億5230万3千", "all": All letter must be Kanji, "auto": detect automatically by checking any arabic character is in kanjis.
    :return: int
    """
    if error not in ("raise", "warn", "ignore"):
        raise ValueError("unexpected value {} for argument error".format(error))
    number_dict = {"一": 1, "二": 2, "三": 3, "四": 4, "五": 5, "六": 6, "七": 7, "八": 8, "九": 9}
    little_digit_dict = {"十": 1, "百": 2, "千": 3}
    digit_dict = {"万": 4, "億": 8, "兆": 12, "京": 16, "垓": 20, "𥝱": 24, "穣": 28, "溝": 32, "澗": 36, "正": 40, "載": 44,
             "極": 48, "恒河沙": 52, "阿僧祇": 56, "那由多": 60, "不可思議": 64, "無量大数": 68}

    if style not in ("all", "mixed", "auto"):
        raise ValueError("unexpected value {} for argument style".format(style))  # check arguments

    result = 0
    if style == "mixed" or (style == "auto" and any(str(num) in kanjis for num in range(10))):
        while kanjis:
            value = 0
            while kanjis:
                left_value = re.compile(rf'(^\d*)(\.\d+)?([千百十]?)(.*)').search(kanjis)
                current, float_number, little_digit,  kanjis = left_value.groups()
                current = int(current or 1)
                if float_number:
                    current += float(float_number)
                if little_digit:
                    current *= 10 ** little_digit_dict[little_digit]
                value += current

                head = re.match(r'{}'.format('|'.join(digit_dict.keys())), kanjis)
                if not kanjis or head:
                    digit = head.group(0) if kanjis else ''
                    result += value * 10 ** digit_dict.get(digit, 0)
                    value = 0
                    kanjis = kanjis[len(digit):]

        return result
    else:
        current_mini_num = 0
        current_num = 0
        for word in re.compile('|'.join(list(number_dict.keys()) + list(little_digit_dict.keys()) + list(digit_dict.keys()))) \
                .findall(kanjis):
            if word in number_dict:
                current_mini_num = number_dict[word]
            elif word in little_digit_dict:
                current_num += (current_mini_num if current_mini_num else 1) * 10 ** little_digit_dict[word]
                current_mini_num = 0
            elif word in digit_dict:
                result += (current_num + current_mini_num) * 10 ** digit_dict[word]
                current_num = current_mini_num = 0
            else:
                raise ValueError("unexpected letter: {}".format(word))
        return result + current_num + current_mini_num


class Number(int):
    @classmethod
    def from_kanji(cls, kanjis, error="raise", style="auto"):
        return cls(kanji2int(kanjis=kanjis, error=error, style=style))

    def to_kanji(self, error="raise", style="all", kanji_thousand=True):
        return int2kanji(number=int(self), error=error, style=style, kanji_thousand=kanji_thousand)

    def __add__(self, other):
        return Number(int.__add__(self, other))

    def __sub__(self, other):
        return Number(int.__sub__(self, other))

    def __mul__(self, other):
        return Number(int.__mul__(self, other))

    def __floordiv__(self, other):
        return Number(int.__floordiv__(self, other))

    def __mod__(self, other):
        return Number(int.__mod__(self, other))

    def __pow__(self, power, modulo=None):
        return Number(int.__pow__(self, power, modulo))

    def __and__(self, other):
        return Number(int.__and__(self, other))

    def __or__(self, other):
        return Number(int.__or__(self, other))

    def __xor__(self, other):
        return Number(int.__xor__(self, other))

    def __lshift__(self, other):
        return Number(int.__lshift__(self, other))

    def __rshift__(self, other):
        return Number(int.__rshift__(self, other))

    def __repr__(self):
        return "Number<{}>".format(int(self))



"""
Easy converter between Kanji-Number and Integer
-----------
Powered by [Yamato Nagata](https://twitter.com/514YJ)

[GitHub](https://github.com/nagataaaas/Kanjize)

Can Convert Number up to 10 ** 72 - 1

```python
from kanjize import int2kanji, kanji2int, Number

print(int2kanji(58076099))
    # 五千八百七万六千九十九

print(kanji2int("五千八百七万六千九十九"))
    # 58076099

print(kanji2int("223兆4千億4256万6千"))
    # 223400042566000

print(kanji2int("223兆4000億4256万6000"))
    # 223400042566000

print(int2kanji(223400042566000, style="mixed", kanji_thousand=False))
    # 223兆4000億4256万6000

print(int2kanji(223400042566000, style="mixed"))
    # 223兆4千億4256万6千

print((Number.from_kanji("223兆4千億4256万6千") * Number(2.3)).to_kanji(style="mixed"))
    # 446兆8千億8513万2千
```
"""

from setuptools import setup
from os import path

about = {}
with open("kanjize/__about__.py") as f:
    exec(f.read(), about)

here = path.abspath(path.dirname(__file__))

setup(name=about["__title__"],
      version=about["__version__"],
      url=about["__url__"],
      license=about["__license__"],
      author=about["__author__"],
      author_email=about["__author_email__"],
      description=about["__description__"],
      long_description=__doc__,
      long_description_content_type="text/markdown",
      packages=["kanjize"],
      zip_safe=False,
      platforms="any",
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Other Environment",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python",
          "Topic :: Software Development :: Libraries :: Python Modules"
      ])
